/* SPDX-License-Identifier: GPL-2.0 WITH Linux-syscall-note */

#ifndef _KSEMEM_H_
#define _KSEMEM_H_	1

#include <linux/types.h>
#include <linux/const.h>
#include "uapi/semem/semem_com.h"
#include "uapi/semem/semem_err.h"

#define SEMEM_MODULE_VERSION	"3.0.1"

extern int secure_env_check(void);

#define OPS_USER_OPEN	0x1ul
#define OPS_USER_CLOSE	0x2ul
#define OPS_USER_CREATE	0x4ul
#define OPS_USER_REMOVE	0x8ul
#define OPS_USER_ATTACH 0x10ul
#define OPS_USER_LISTEN	0x20ul

#define OPS_CREATE		0x10000ul
#define OPS_REMOVE		0x20000ul
#define OPS_ACQUIRE		0x40000ul
#define OPS_RELEASE		0x80000ul
#define OPS_LISTEN		0x100000ul
#define OPS_STATUS		0x200000ul

#define OPS_NOSYNC		(1ul << 32)

/*
* common definition of secure key
*/
struct dimm_key_struct {
	unsigned int key_low;
	unsigned int key_high;
};

struct ksemem_work;
struct task_struct;
extern struct ksemem_work *ksemem_dummy_work_register(
				bool period, char *name,
				long (*exit)(struct ksemem_work *, long));
extern long ksemem_work_name(struct ksemem_work *work, char *name, long len);
extern bool ksemem_work_stop(struct ksemem_work *work);

extern long ksemem_global_timer_start(long interval);
extern long ksemem_global_timer_stop(bool sync, void (*__cb)(long));
extern long ksemem_global_timer_update(long interval);
extern long ksemem_global_timer_interval(void);

extern long ksemem_fs_rule_create(unsigned long pa, unsigned long size,
				unsigned long attr, unsigned long status,
				void *arg);
extern long ksemem_fs_rule_acquire(unsigned long pa, unsigned long size,
				void *arg);
extern long ksemem_fs_rule_release(unsigned long pa, unsigned long size,
				void *arg);
extern long ksemem_fs_rule_status_update(unsigned long pa, unsigned long size,
				unsigned long status, void *arg);
extern long ksemem_fs_device_init(long (*store)(long, void *),
				long (*load)(long, void *));
extern long ksemem_fs_device_ksync(long (*store)(long, void *));
extern long ksemem_fs_device_kload(long (*load)(long, void *));

extern long ksemem_fs_validate_proc_register(void *validate_proc,
				void *validate_proc_start,
				void *validate_proc_finish);

extern long prototype_validate_proc(struct task_struct *proc, long op,
				void *arg);
extern long prototype_validate_proc_start(struct task_struct *task, long op,
				void *buf, long size, void *arg);
extern long prototype_validate_proc_finish(struct task_struct *task, long op,
				void *buf, long size, void *arg);

extern int ksemem_system_range_decode(unsigned long sys_start,
				unsigned long sys_end,
				struct list_head *rcd_rule_head);

extern int rcd_init_key(struct dimm_key_struct keys[]);
extern int rcd_update_random_key(struct dimm_key_struct keys[]);
extern int rcd_sync_random_key(struct dimm_key_struct keys[]);
#endif
