/*
 *  Kylin security devctl control
 *
 *  This file contains the Kylin security devctl function
 *      implementations.
 *
 *  Authors:  wangkanglong <wangkanglong@kylinos.com.cn>
 *            yangzhao, <yangzhao@kylinos.cn>
 *
 *  Copyright (C) 2012-2021 Kylin of GFKD
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License version 2,
 *      as published by the Free Software Foundation.
 */

#ifndef __KYSEC_DEVCTL_INCLUDE_H
#define __KYSEC_DEVCTL_INCLUDE_H

enum kysec_dev_type {
	KYSEC_USB = 1,
	KYSEC_SERIAL,
	KYSEC_PARALLEL,
	KYSEC_TYPEC,
	KYSEC_CDROM,
	KYSEC_PRINTER,
	KYSEC_WIRELESS,
	KYSEC_ETHERNET,
	KYSEC_BLUETOOTH,
	KYSEC_CAMERA,
	KYSEC_SCANNER,
	KYSEC_AUDIO,
	KYSEC_IEEE1394,
	KYSEC_PCMCIA,
	KYSEC_PCIE,
	KYSEC_SATA,
	KYSEC_PS2,
	KYSEC_VGA,
	KYSEC_HDMI,
	KYSEC_DP,
	KYSEC_INFRARED,
	KYSEC_MODEM,
	KYSEC_MMC,
	KYSEC_SCSI_RAID,
	KYSEC_FLOPPY_DRIVE,
	KYSEC_VIDEO,
	KYSEC_DIGITAL_CAMERA,
	KYSEC_MULTI_SERIAL_CARD,
	KYSEC_WINCE,
	KYSEC_PRINTER_LOCAL,
	KYSEC_PRINTER_NET,
	KYSEC_FILE_SHARING,
	KYSEC_BATTERY_AND_UPS,
	KYSEC_BIOMETRIC,
	KYSEC_DISK_DRIVE,
	KYSEC_DISPLAY_CARD,
	KYSEC_HDC,
	KYSEC_KEYBOARD,
	KYSEC_MOUSE,
	KYSEC_STORAGE_VOLUME,
	KYSEC_CDDRIVE,
	KYSEC_VOLUME_SHADOW,
	KYSEC_TAPE,
	KYSEC_DEV_MAX
};

enum kysec_usb_type {
	KYSEC_USB_STORAGE = 1,
	KYSEC_USB_CDROM,
	KYSEC_USB_PRINTER,
	KYSEC_USB_CAMERA,
	KYSEC_USB_BLUETOOTH,
	KYSEC_USB_MASS_STORAGE,
	KYSEC_USB_PER_INTERFACE,
	KYSEC_USB_AUDIO,
	KYSEC_USB_COMM,
	KYSEC_USB_HID,
	KYSEC_USB_PHYSICAL,
	KYSEC_USB_STILL_IMAGE,
	KYSEC_USB_HUB,
	KYSEC_USB_CDC_DATA,
	KYSEC_USB_CSCID,
	KYSEC_USB_CONTENT_SEC,
	KYSEC_USB_MISC,
	KYSEC_USB_APP_SPEC,
	KYSEC_USB_VENDOR_SPEC,
	KYSEC_USB_PERSONAL_HEALTH,
	KYSEC_USB_DIAGNOSTIC,
	KYSEC_USB_MAX
};

/* device permission */
#define KYSEC_DEV_PERM_DEFAULT 0
#define KYSEC_DEV_PERM_ENABLE 1
#define KYSEC_DEV_PERM_DISABLE 2

/* 5 = 4 | KYSEC_DEV_PERM_ENABLE */
#define KYSEC_DEV_PERM_RO 5
#define KYSEC_DEV_PERM_RW KYSEC_DEV_PERM_ENABLE
#define KYSEC_DEV_PERM_BURN KYSEC_DEV_PERM_ENABLE


/* func: get device permission from kysec devctl module
 * @dev_type: is device type, which is the value of kysec_dev_type
 * @dev: is struct device pointer
 *
 * return: the true permission
 */

struct device;
struct block_device;
struct net_device;
struct usb_interface;
struct usb_device;
struct usb_driver;
struct pci_dev;

#ifdef CONFIG_SECURITY_KYSEC_DEVICE_CONTROL
int kysec_devctl_do_mount(const char *dev_name, unsigned long *flags);
int kysec_devctl_cdrom_has_perm(struct block_device *bdev, unsigned int cmd,
			void __user *argp);
int kysec_devctl_has_perm(int dev_type, struct device *dev);
int kysec_devctl_net_permission(struct net_device *ndev);
int kysec_devctl_printer_permission(struct usb_interface *uif, int usb_bool);
int kysec_devctl_drm_permission(int connector_type);
int kysec_devctl_get_pci_permission(int dev_type, struct pci_dev *dev);
int kysec_devctl_device_has_perm(struct device *dev);

bool kysec_devctl_get_sm_status(void);
int kysec_devctl_add_usbdevinfo(struct device *dev, struct usb_driver *driver,
		struct usb_interface *intf);
void kysec_devctl_remove_usbdevinfo(struct usb_device *udev);
void kysec_devctl_notify_user(int plug, unsigned int pid,
		unsigned int vid, int perm, char *serial,
		char *manufacture, char *product);

#else
static inline int kysec_devctl_do_mount(const char *dev_name, unsigned long *flags)
{
	return 0;
}

static inline int kysec_devctl_cdrom_has_perm(struct block_device *bdev, unsigned int cmd,
			void __user *argp)
{
	return 0;
}

static inline int kysec_devctl_has_perm(int dev_type, struct device *dev)
{
	return 0;
}

static inline int kysec_devctl_net_permission(struct net_device *ndev)
{
	return 0;
}

static inline int kysec_devctl_printer_permission(struct usb_interface *uif, int usb_bool)
{
	return 0;
}

static inline int kysec_devctl_drm_permission(int connector_type)
{
	return 0;
}

static inline int kysec_devctl_get_pci_permission(int dev_type, struct pci_dev *dev)
{
	return 0;
}

static inline int kysec_devctl_add_usbdevinfo(struct device *dev, struct usb_driver *driver,
		struct usb_interface *intf)
{
	return 0;
}

static inline void kysec_devctl_remove_usbdevinfo(struct usb_device *udev)
{
	return;
}

static inline void kysec_devctl_notify_user(int plug, unsigned int pid,
		unsigned int vid, int perm, char *serial,
		char *manufacture, char *product)
{

}

static inline int kysec_devctl_device_has_perm(struct device *dev)
{
	return 0;
}

#endif /* CONFIG_SECURITY_KYSEC_DEVICE_CONTROL */
#endif
