#!/bin/bash

ARCH=`uname -m`
Arch=`echo $ARCH | sed -e s/i.86/x86/ -e s/x86_64/x86/ -e s/amd64/x86/ -e s/aarch64.*/arm64/ -e s/mips64el/mips/ -e s/loongarch64.*/loongarch/`

usage() {
	echo -e "Usage: `basename $0` [OPTION]..."
	echo ""
	echo -e "  -h, --help\t\tShow This Help"
	echo -e "  -c, --config\t\tBuild Test Kernel With Specify Config"
	echo -e "  -l, --label\t\tBuild Kernel With Specify Label"
	echo -e "  -S, --source\t\tCreate kernel source rpm package"
	echo -e "  --baseonly\t\tBuild Base Kernel Only"
	echo -e "  --bivonly\t\tBuild Benchmark Improve Version Only"
	echo -e "  --debugonly\t\tBuild Debug Kernel Only"
if [ ${Arch}x = "arm64"x ]; then
	echo -e "  --4k\t\t\tBuild 4k pagesize kernel only"
fi
	echo -e "  --full\t\tBuild Full Kernel"
	echo -e "  --without-kabi-check\tBuild without kabi check"
	echo -e "  --without-test\tBuild without .fortest label"
if [ ${Arch}x = "mips"x -o ${Arch}x = "loongarch"x ]; then
	echo -e "  --kdump\t\tBuild Kernel Used By Kdump"
fi
	echo ""
	echo "Examples:"
	echo "  ./scripts/buildrpm.sh --baseonly"
}

if [ $# -lt 1 ]; then
	usage
	exit -1;
fi

create-tarball() {
	GITID=$1
	TARBALL=$2
	DIR=$3

	if [ ! -z $4 ]; then
		ADD_FILE="--add-file .sourceversion"
	fi

	# shellcheck disable=SC1083
	XZ_THREADS=$(rpm --eval %{_smp_mflags} | sed -e 's!^-j!--threads !')

	ARCH=$(arch)
	XZ_OPTIONS=""

	if [ -f "$TARBALL" ]; then
		TARID=$(xzcat -qq "$TARBALL" | git get-tar-commit-id 2>/dev/null)
		GITID_NORMALIZE=$(git log --max-count=1 --pretty=format:%H "$GITID")
		if [ "${GITID_NORMALIZE}" = "${TARID}" ]; then
			echo "$(basename "$TARBALL") unchanged..."
			exit 0
		fi
		rm -f "$TARBALL"
	fi

	echo "Creating $(basename "$TARBALL")..."
	trap 'rm -vf "$TARBALL"' INT
	# XZ_OPTIONS and XZ_THREADS DEPEND on word splitting, so don't disable it here:
	# shellcheck disable=SC2086
	git archive --prefix="$DIR"/ --format=tar "$GITID" ${ADD_FILE} | xz $XZ_OPTIONS $XZ_THREADS > "$TARBALL";
}

BUILDUP=false
BUILDBIV=false
BUILDDEBUG=false
BUILD4K=false
BUILDKDUMP=false
BUILD_WITH_KABICHECK=true
BUILD_NOARCH=false
#do building
BUILD_ARGS=""
TESTLAB=".fortest"

ARGS=$(getopt -o hc:l:S -a --long help,without-test,baseonly,bivonly,debugonly,full,4k,config:,label:,kdump,without-kabi-check,source -- "$@")
if [ $? != 0 ]; then
	echo "Terminating..."
	usage && exit 1
fi

eval set -- "${ARGS}"

while true
do
	case $1 in
		--baseonly)
			BUILDUP=true
			shift;;
		--bivonly)
			BUILDBIV=true
			shift;;
		--debugonly)
			BUILDDEBUG=true
			shift;;
		--full)
			BUILDUP=true
			BUILDBIV=true
			BUILD4K=true
			BUILDDEBUG=true
			BUILDKDUMP=true
			BUILD_NOARCH=true
			# I'm a kernel option for release, so there is no need
			# build with '.fortest' tag.
			TESTLAB=""
			shift;;
		--4k)
			BUILD4K=true
			shift;;
		--without-test)
			TESTLAB=""
			shift;;
		-l|--label)
			TESTLAB=".$2"
			shift 2;;
		-S|--source)
			CREATE_SOURCE_PKG="-bs"
			shift;;
		--without-kabi-check)
			BUILD_ARGS="--without kabichk"
			BUILD_WITH_KABICHECK=false
			shift;;
		--)
			shift
			break;;
		--kdump)
			if [ ${Arch}x = "mips"x -o ${Arch}x = "loongarch"x ]; then
				BUILDUP=true
				BUILDBIV=false
				BUILDDEBUG=false
				BUILDKDUMP=true
				TESTLAB=""
			else
				echo "Option --kdump only supports mips and loongarch."
				exit -1
			fi
			shift;;
		-h|--help|*)
			usage
			exit -1;;
	esac
done

for arg in $@
do
	echo "Ignore Unknow Args $arg.."
	usage && exit -1
done

## Ensure BUILD4k only effect on aarch64
if [[ x"$Arch" != x"arm64" ]]; then
	BUILD4K=false
fi

## Ensure BUILDKDUMP only effect on loongarch or mips
if [ ${Arch}x != "mips"x -a ${Arch}x != "loongarch"x ]; then
	BUILDKDUMP=false
fi

if [[ $BUILDBIV == false ]]; then
	BUILD_ARGS+=" --without biv"
fi

if [[ $BUILDDEBUG == false ]]; then
	BUILD_ARGS+=" --without debug"
fi

if [[ $BUILDUP == false ]]; then
	BUILD_ARGS+=" --without up"
fi

if [[ $BUILDKDUMP == true ]]; then
	BUILD_ARGS+=" --with kdump"
fi

if [[ $BUILD4K == true ]]; then
	BUILD_ARGS+=" --with 4k"
fi

git update-index --refresh
git diff-index --quiet HEAD --

if [[ $? != 0 ]]; then
	HAS_UNSTAGE=1
	git commit -as -m "test"
fi

GITID=$(git rev-parse HEAD)
TARFILE=linux-4.19.90.tar.xz
TARBALL=rpmbuild/SOURCES/$TARFILE
RPMBUILD=`pwd`/rpmbuild

## clean latest time rpms
rm -rf $RPMBUILD/RPMS

echo ${GITID} > .sourceversion
create-tarball $GITID $TARBALL linux-4.19.90 ${CREATE_SOURCE_PKG}
rpmbuild ${CREATE_SOURCE_PKG:--ba} --define "_sourcedir $RPMBUILD/SOURCES" --define "_builddir $RPMBUILD/BUILD" \
	     --define "_srcrpmdir $RPMBUILD/SRPMS" --define "_rpmdir $RPMBUILD/RPMS" \
	     --define "_buildrootdir $RPMBUILD/BUILDROOT" \
	     --define "_specdir $RPMBUILD/SPECS" rpmbuild/SPECS/kernel.spec \
	     ${BUILD_ARGS} ${TESTLAB:+--define} ${TESTLAB:+"__custom $TESTLAB"}

if [ $? -ne 0 ]; then
	echo "building kernel failed..."
	exit
fi

if [ ! -z $CREATE_SOURCE_PKG ]; then
	exit;
fi

if [[ $BUILD_NOARCH == true ]]; then
	rpmbuild -bb --target=noarch --define "_sourcedir $RPMBUILD/SOURCES" \
		 --define "_builddir $RPMBUILD/BUILD" \
		 --define "_srcrpmdir $RPMBUILD/SRPMS" --define "_rpmdir $RPMBUILD/RPMS" \
		 --define "_buildrootdir $RPMBUILD/BUILDROOT" \
		 --define "_specdir $RPMBUILD/SPECS" rpmbuild/SPECS/kernel.spec
fi

#clean work
if [ $? -eq 0 ]; then
	echo "building rpm success..."
	rm -rf rpmbuild/kernel
else
	echo "building failed..."
	exit
fi

#archive RPMS
MIDV=`grep -rn "midv" rpmbuild/SPECS/kernel.spec  | head  -n 1  | awk -F' ' '{print $3}'`
SUBV=`grep -rn "subv" rpmbuild/SPECS/kernel.spec  | head  -n 1  | awk -F' ' '{print $3}'`
RPMDIR=`pwd`/rpmbuild/RPMS/${ARCH}
KVRELEASE=kernel-4.19.90-${MIDV}.${SUBV}${TESTLAB}-${ARCH}
rm -rf rpmbuild/$KVRELEASE*
mkdir -p rpmbuild/$KVRELEASE

for pack in `ls $RPMDIR/${rpmpkg}*.rpm | grep -v debuginfo-common`
do
	cp $pack rpmbuild/$KVRELEASE
done

if [[ $BUILDBIV != false ]]; then
	mkdir -p rpmbuild/$KVRELEASE/biv-version
	mv rpmbuild/$KVRELEASE/*biv*.rpm rpmbuild/$KVRELEASE/biv-version
	VERSIONS+=" biv-version"
fi

if [[ $BUILDDEBUG != false ]]; then
	mkdir -p rpmbuild/$KVRELEASE/debug-version
	mv rpmbuild/$KVRELEASE/*kernel-debug-*.rpm rpmbuild/$KVRELEASE/debug-version
	VERSIONS+=" debug-version"
fi

if [[ $BUILD4K != false ]]; then
	mkdir -p rpmbuild/$KVRELEASE/4k-version
	mv rpmbuild/$KVRELEASE/*4k*.rpm rpmbuild/$KVRELEASE/4k-version
	VERSIONS+=" 4k-version"
fi

if [[ $BUILDKDUMP != false ]]; then
	mkdir -p rpmbuild/$KVRELEASE/kdump-version
	mv rpmbuild/$KVRELEASE/*kernel-kdump-*.rpm rpmbuild/$KVRELEASE/kdump-version
	VERSIONS+=" kdump-version"
fi

mkdir -p rpmbuild/$KVRELEASE/all
mv rpmbuild/$KVRELEASE/{*tool*,*perf*,kernel-cross-headers*} rpmbuild/$KVRELEASE/all

if [[ $BUILD_NOARCH == true ]]; then
	mv rpmbuild/RPMS/noarch/*.rpm rpmbuild/$KVRELEASE/all
fi

if [[ $BUILDUP != false ]]; then
	mkdir -p rpmbuild/$KVRELEASE/base-version
	mv rpmbuild/$KVRELEASE/*.rpm rpmbuild/$KVRELEASE/base-version
	VERSIONS+=" base-version"
fi

for version in $VERSIONS
do
	cp rpmbuild/$KVRELEASE/all/* rpmbuild/$KVRELEASE/$version
	cp rpmbuild/SOURCES/readme-kernel-rpm.txt rpmbuild/$KVRELEASE/$version
	echo "This is a $version kernel package: 4.19.90-${MIDV}.${SUBV}-${ARCH}-${version}" > rpmbuild/$KVRELEASE/$version/README
	echo "For details about kernel rpm and usage, please refer to readme-kernel-rpm.txt!" > rpmbuild/$KVRELEASE/$version/README
	pushd rpmbuild/$KVRELEASE/
	tar -zcvf ${version}.tar.gz $version
	rm -rf $version
	popd
done

rm -rf rpmbuild/$KVRELEASE/all

#clean
if [[ $HAS_UNSTAGE == 1 ]]; then
	git reset HEAD^
fi

pushd rpmbuild
tar -zcvf ${KVRELEASE}.tar.gz $KVRELEASE
rm -rf $KVRELEASE
echo -e "Archive all builded rpm packages in: \nrpmbuild/${KVRELEASE}.tar.gz"
popd
