#!/bin/bash

BOK="\033[32mOK\033[0m"
BFAIL="\033[31mFAIL\033[0m"
BPASS="\033[33mnocheck\033[0m"
BUILD_RES="\nBuilding result:\n\tArch\t\tcompile\t\tkabi"

ncpus=$(nproc)

function usage()
{
	echo -e "`basename $0` [OPTION]..."
	echo -e "\t$(basename $0) will compile and check the KABI for the specified arch, if no arch specified it will only build for local arch!"

	echo ""
	echo -e "  -h, --help\t\t\tshow this help info"
	echo -e "  -a, --all\t\t\tcompile and check kabi for x86,arm64 and mips"
	echo -e "  -A, --arm64\t\t\tcompile and check kabi for arm64"
	echo -e "  -X, --x86_64\t\t\tcompile and check kabi for x86_64"
	echo -e "  -M, --mips\t\t\tcompile and check kabi for mips64el"
	echo -e "  -L, --loongarch64\t\tcompile and check kabi for loongarch"

}

# need a parameter to specify arch
function compile_and_chkabi_for_target()
{

	if [ x"$1" != "xx86_64" ] && [ x"$1" != "xmips" ] && [ x"$1" != "xaarch64" ] && [ x"$1" != "xloongarch64" ];then
		echo "please specify target arch to only one of x86_64, mips, aarch64!"
		return 1
	fi

	if [ "$(uname -m | sed 's/mips64el/mips/')" == "$1" ]; then
		if [ $(which gcc &> /dev/null; echo $?) -ne 0 ]; then
			echo " No gcc compiler found ,please install firstly!"
			BUILD_RES="${BUILD_RES}\n\t${1}\t\t${BPASS}\t\t${BPASS}"
			return 1;
		fi
		cross_arg="-j ${ncpus}"
	elif [ $(which ${1}-linux-gnu-gcc &> /dev/null; echo $?) -ne 0 ]; then
		echo " cann't do cross compile for $1, no cross compile found!"
		echo " please install $1-linux-gnu- series toolchain."
		BUILD_RES="${BUILD_RES}\n\t${1}\t\t${BPASS}\t\t${BPASS}"
		return 1
	else
		if [ $1 == loongarch64 ];then
			cross_arg="CROSS_COMPILE=$1-linux-gnu- ARCH=loongarch -j ${ncpus}"
		else
			cross_arg="CROSS_COMPILE=$1-linux-gnu- ARCH=$1 -j ${ncpus}"
		fi
	fi

	make $cross_arg kylinos_generic_defconfig && make $cross_arg

	if [ $? -ne 0 ]; then
		echo " cross compile failed for target $1!"
		BUILD_RES="${BUILD_RES}\n\t${1}\t\t${BFAIL}\t\t${BPASS}"
		return 1;
	fi

	echo " cross compile success for target $1!"

	./scripts/check-kabi  -k rpmbuild/SOURCES/Module.kabi_$(echo $1 | sed 's/mips/mips64el/')  -s Module.symvers

	if [ $? -ne 0 ]; then
		echo " kabi check for target $1 failed!"
		BUILD_RES="${BUILD_RES}\n\t${1}\t\t${BOK}\t\t${BFAIL}"
		return 2;
	fi

	echo " kabi check for target $1 success!"

	BUILD_RES="${BUILD_RES}\n\t${1}\t\t${BOK}\t\t${BOK}"
	return 0
}

HAS_TARGET="false"
DO_X86_64="false"
DO_ARM64="false"
DO_MIPS="false"
DO_LOONGARCH="false"

case `uname -m` in
	x86_64)
		local_arch="x86_64";;
	aarch64)
		local_arch="aarch64";;
	loongarch64)
		local_arch="loongarch64";;

	*)
		echo "Currently only support x86 and arm64 yet!";
		exit 1;;
esac


ARGS=`getopt -o ahXAML -a --long help,x86_64,arm64,mips,loongarch64,all -- "$@"`
if [ $? != 0 ]; then
        echo "Terminating..."
        usage && exit 1
fi

eval set -- "${ARGS}"

while true
do
        case $1 in
                -h|--help)
                        usage
                        exit 0;;
                -a|--all)
			DO_X86_64="true"
			DO_ARM64="true"
			DO_MIPS="true"
			DO_LOONGARCH="true"
			HAS_TARGET="true"
                        shift 1;;
                -X|--x86_64)
			DO_X86_64="true"
			HAS_TARGET="true"
                        shift 1;;
                -A|--arm64)
			DO_ARM64="true"
			HAS_TARGET="true"
                        shift 1;;
		-M|--mips)
			DO_MIPS="true"
			HAS_TARGET="true"
                        shift 1;;
		-L|--loongarch64)
			DO_LOONGARCH="true"
			HAS_TARGET="true"
			shift 1;;
		--)
			shift
			break;;
	esac
done

# defaultly only do for local arch
if [ "$HAS_TARGET" == "false" ]; then
	compile_and_chkabi_for_target $(uname -m | sed 's/mips64el/mips/')
fi

if [ "$DO_ARM64" == "true" ]; then
	compile_and_chkabi_for_target "aarch64"
fi

if [ "$DO_X86_64" == "true" ]; then
	compile_and_chkabi_for_target "x86_64"
fi

if [ "$DO_MIPS" == "true" ]; then
	compile_and_chkabi_for_target "mips"
fi

if [ "$DO_LOONGARCH" == "true" ]; then
	compile_and_chkabi_for_target "loongarch64"
fi

echo -e ${BUILD_RES}
