#!/bin/bash

# linux_headers_install.sh
#
# Simple script to extract linux headers and friends needed to build
# out of tree modules without having the all source tree around.
#
# Reference scripts/package/builddeb

ARCH=$1
hdr_src_files=$objtree/hdrsrcfiles
hdr_obj_files=$objtree/hdrobjfiles
version=`cat include/config/kernel.release`
linux_headers=linux-headers-$version

mkdir -p $linux_headers

#build list of headers and friends
(cd $srctree; find . -name Makefile\* -o -name Kconfig\* -o -name \*.pl) \
        > "$hdr_src_files"
(cd $srctree; find arch/$SRCARCH/include include scripts -type f) \
        >> "$hdr_src_files"
(cd $srctree; find arch/$SRCARCH -name module.lds -o -name Kbuild.platforms \
        -o -name Platform) >> "$hdr_src_files"
(cd $srctree; find $(find arch/$SRCARCH -name include -o -name scripts \
        -type d) -type f) >> "$hdr_src_files"
(cd $srctree; find $(find security/selinux tools/include -type f) -name *.h) \
	>> "$hdr_src_files"
(cd $objtree; find arch/$SRCARCH/include Module.symvers include scripts \
        -type f) > "$hdr_obj_files"

(cd $srctree; tar -c -f - -T -) < "$hdr_src_files" | (cd $linux_headers; tar -xf -)
(cd $objtree; tar -c -f - -T -) < "$hdr_obj_files" | (cd $linux_headers; tar -xf -)
(cd $objtree; cp $KCONFIG_CONFIG $linux_headers/)
(cd $objtree; tar -czf $linux_headers.tar.gz $linux_headers )


# clean
rm -f $hdr_src_files
rm -f $hdr_obj_files
rm -rf $objtree/$linux_headers

# Build linux_headers-$version.sh.
# On target machine, Running ./linux_header-$version.sh will install linux headers automatically.
(cd $objtree; cat > $linux_headers.sh)  << EOF
#!/bin/bash
dst=/usr/src
archive_path=\$(realpath "\$0")
sed -e '1,/^exit_marker$/d' \$archive_path | tar -xzf - -C \$dst

[ "$ARCH" != "`uname -m`" ] && (cd \$dst/$linux_headers; make -s scripts_basic; make -s M=scripts/mod; make -s M=scripts)

rm /lib/modules/$version/build
ln -s \$dst/$linux_headers /lib/modules/$version/build

exit 0
exit_marker
EOF
chmod +x $linux_headers.sh
(cd $objtree; cat $linux_headers.tar.gz >> $linux_headers.sh)